{% load i18n %}

<dl>
    <dt class="nav-header">{{ name }}</dt>
    {% for item in items %}
        <dd>
            {% if item.selected %}
                <input class="facet_checkbox" type="checkbox" name="{{ item.name }}" checked />
                <input class="facet_url" type="hidden" name="url_for_{{ item.name }}" value="{{ item.deselect_url }}" />
                <strong>{{ item.name }}</strong>
            {% else %}
                {% if item.disabled %}
                    <input type="checkbox" name="{{ item.name }}" disabled />
                    {{ item.name }}
                {% else %}
                    <input class="facet_checkbox" type="checkbox" name="{{ item.name }}" />
                    <input class="facet_url" type="hidden" name="url_for_{{ item.name }}" value="{{ item.select_url }}" />
                    <a href="{{ item.select_url }}">{{ item.name }}</a>
                {% endif %}
            {% endif %}
            {% if item.show_count %}
                ({{ item.count }})
            {% endif %}
        </dd>
    {% endfor %}
</dl>