# Generated by Django 5.2.4 on 2025-07-15 12:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("catalogue", "0031_productcategoryhierarchy"),
    ]

    operations = [
        migrations.AddField(
            model_name="category",
            name="exclude_from_menu",
            field=models.BooleanField(
                db_index=True,
                default=False,
                help_text="Exclude this category from the menu.",
                verbose_name="Exclude from menu",
            ),
        ),
        migrations.AddField(
            model_name="category",
            name="long_description",
            field=models.TextField(blank=True, verbose_name="Long description"),
        ),
    ]
