{% extends 'oscar/layout.html' %}
{% load i18n %}

{% block title %}{% trans 'Password reset email sent' %} | {{ block.super }}{% endblock %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item">
                <a href="{{ homepage_url }}">{% trans 'Home' %}</a>
            </li>
            <li class="breadcrumb-item">
                <a href="{% url 'password-reset' %}">{% trans 'Password reset' %}</a>
            </li>
            <li class="breadcrumb-item active" aria-current="page">{% trans 'Email sent' %}</li>
        </ol>
    </nav>
{% endblock %}

{% block headertext %}{% trans "Email sent" %}{% endblock %}

{% block content %}
    <p>
    {% blocktrans %}
        We've e-mailed you instructions for resetting your password. You should be receiving it shortly.
    {% endblocktrans %}
    </p>
    <p>
    {% blocktrans %}
        If you don't receive an email, please make sure you've entered the address you registered with, and check
        your spam folder.
    {% endblocktrans %}
    </p>
{% endblock %}
