{% load currency_filters %}
{% load i18n %}

<div class="basket-mini col-sm-5 text-right d-none d-md-block">
    <strong>{% trans "Basket total:" %}</strong>
    {% if request.basket.is_tax_known %}
        {{ request.basket.total_incl_tax|currency:request.basket.currency }}
    {% else %}
        {{ request.basket.total_excl_tax|currency:request.basket.currency }}
    {% endif %}

    <div class="btn-group">
      <button type="button" class="{% block mini_basket_btn_classes %}btn btn-outline-secondary{% endblock %}" onclick="window.location.href='{% url 'basket:summary' %}';">{% trans "View basket" %}</button>
      <button type="button" class="{% block mini_basket_toggle_classes %}btn btn-outline-secondary dropdown-toggle dropdown-toggle-split{% endblock %}" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
        <span class="sr-only">Toggle Dropdown</span>
      </button>
      <div class="{% block mini_basket_dropdown_classes %}dropdown-menu dropdown-menu-right{% endblock %}">
        {% include "oscar/basket/partials/basket_quick.html" %}
      </div>
    </div>
</div>
