{% extends 'oscar/layout.html' %}
{% load i18n %}

{% block title %}{% trans 'Password reset' %} | {{ block.super }}{% endblock %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item">
                <a href="{{ homepage_url }}">{% trans 'Home' %}</a>
            </li>
            <li class="breadcrumb-item active" aria-current="page">{% trans 'Password reset' %}</li>
        </ol>
    </nav>
{% endblock %}

{% block headertext %}{% trans "Password reset" %}{% endblock %}

{% block content %}

    <form id="password_reset_form" method="post">
        {% csrf_token %}
        <p>{% trans "Forgotten your password? Enter your e-mail address below, and we'll e-mail instructions for setting a new one." %}</p>
        {% include 'oscar/partials/form_fields.html' with style='stacked' %}
        <div class="form-group form-actions">
            <button type="submit" class="btn btn-primary btn-lg" data-loading-text="{% trans 'Sending...' %}">{% trans 'Send reset email' %}</button>
        </div>
    </form>

{% endblock %}
