{% load i18n %}

{% block message_content %}
{% blocktrans with name=offer.name %}
Your basket now qualifies for the 
<strong>{{ name }}</strong> offer.
{% endblocktrans %}
{% endblock message_content %}

{% block analytics_scripts %}{% endblock analytics_scripts %}
