# Generated by Django 3.2.19 on 2023-07-10 08:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0013_json_option_value'),
    ]

    operations = [
        migrations.AddField(
            model_name='line',
            name='tax_code',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='VAT rate code'),
        ),
        migrations.AddField(
            model_name='lineprice',
            name='tax_code',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='VAT rate code'),
        ),
        migrations.AddField(
            model_name='order',
            name='shipping_tax_code',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='Shipping VAT rate code'),
        ),
        migrations.AddField(
            model_name='surcharge',
            name='tax_code',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='VAT rate code'),
        ),
    ]
