{% extends 'oscar/dashboard/layout.html' %}

{% load i18n %}

{% block body_class %}{{ block.super }} create-page pages{% endblock %}
{% block title %}
{{ title }} | {{ block.super }}
{% endblock %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item"><a href="{% url 'dashboard:index' %}">{% trans "Dashboard" %}</a></li>
            <li class="breadcrumb-item"><a href="{% url 'dashboard:page-list' %}">{% trans "Pages" %}</a></li>
            <li class="breadcrumb-item active" aria-current="page">{{ title }}</li>
        </ol>
    </nav>
{% endblock %}

{% block headertext %}
{{ title }}
{% endblock %}

{% block dashboard_content %}
<div class="table-header">
    <h2>{{ title }}</h2>
</div>

<form id="create_update_page_form" method="post" class="card card-body form-stacked wysiwyg" enctype="multipart/form-data">
    {% csrf_token %}
    {% include 'oscar/dashboard/partials/form_fields.html' with form=form %}
    <div class="form-actions">
        <button class="btn btn-primary" type="submit" data-loading-text="{% trans 'Saving...' %}">{% trans "Save" %}</button>
        {% trans "or" %} <a href="{% url 'dashboard:page-list' %}">{% trans "cancel" %}</a>.
    </div>
</form>

{% endblock dashboard_content %}
