{% extends "oscar/layout.html" %}
{% load i18n %}

{% block title %}
    {% trans 'Login or register' %} | {{ block.super }}
{% endblock %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item">
                <a href="{{ homepage_url }}">{% trans 'Home' %}</a>
            </li>
            <li class="breadcrumb-item active" aria-current="page">{% trans 'Login or register' %}</li>
        </ol>
    </nav>
{% endblock %}

{% block header %}{% endblock %}

{% block content %}

    <div class="row">
        <div class="col-sm-6 login_form">
            <form id="login_form" action="{{ request.get_full_path }}" method="post" class="card card-body bg-light" >
                <h2>{% trans 'Log In' %}</h2>
                {% csrf_token %}
                {% include "oscar/partials/form_fields.html" with form=login_form %}
                <p><a href="{% url 'password-reset' %}">{% trans "I've forgotten my password" %}</a></p>
                <button name="login_submit" type="submit" value="Log In" class="btn btn-lg btn-primary" data-loading-text="{% trans "Logging in..." %}">{% trans 'Log In' %}</button>
            </form>
        </div>
        <div class="col-sm-6 register_form">
            <form id="register_form" action="{{ request.get_full_path }}" method="post" class="card card-body bg-light">
                <h2>{% trans 'Register' %}</h2>
                {% csrf_token %}
                {% include "oscar/partials/form_fields.html" with form=registration_form %}
                <button name="registration_submit" type="submit" value="Register" class="btn btn-lg btn-primary" data-loading-text="{% trans 'Registering...' %}">{% trans 'Register' %}</button>
            </form>
        </div>
    </div>

{% endblock content %}
