{% extends "oscar/communication/emails/base.html" %}
{% load i18n %}
{% load url_tags %}

{% block tbody %}
    <tr>
        <td class="content-block">
            {% blocktrans with name=site.name %}You're receiving this e-mail because your password has been changed at {{ name }}.{% endblocktrans %}
        </td>
    </tr>

    <tr>
        <td class="content-block">
            {% trans "If it wasn't you who changed it, please reset your password immediately:" %}
        </td>
    </tr>

    <tr>
        <td class="content-block">
            <a href="{% absolute_url site.domain reset_url %}" class="btn-primary">{% trans "Reset password" %}</a>
        </td>
    </tr>

    <tr>
        <td class="content-block">
            {% trans "Otherwise, you can ignore this email." %}
        </td>
    </tr>
{% endblock %}
