# Generated by Django 2.0.7 on 2018-11-15 19:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('customer', '0004_email_save'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='date_sent',
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
    ]
