{% load i18n %}
<li class='sortable-handle'>
    {% for field in form %}
        {% if field.is_hidden %}
            {{ field }}
        {% elif "original" in field.id_for_label %}
            <div>{{ field }}</div>
            {% for error in field.errors %}
                <ul class="error-block">
                    <li>{{ error }}</li>
                </ul>
            {% endfor %}
            <a href="#" class="btn btn-info btn-handle btn-reorder {% if not form.instance.id %}disabled{% endif %}" {% if not form.instance.id %}disabled{% endif %}><i class="fas fa-arrows-alt"></i> {% trans "Re-order" context "Change the sequence order" %}</a><br>
        {% else %}
            {{ field.label_tag }}
            {{ field }}
            {% for error in field.errors %}
                <ul class="error-block">
                    <li>{{ error }}</li>
                </ul>
            {% endfor %}
        {% endif %}
    {% endfor %}
    {{ form.non_field_errors }}
</li>
