{% extends "oscar/layout.html" %}
{% load i18n %}

{% block title %}
    {{ review.title }} | {% trans "Reviews" %} | {{ product.title }} | {{ block.super }}
{% endblock %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item">
                <a href="{{ homepage_url }}">{% trans "Home" %}</a>
            </li>
            {% with category=product.categories.all.0 %}
                {% for c in category.get_ancestors_and_self %}
                    <li class="breadcrumb-item">
                        <a href="{{ c.get_absolute_url }}">{{ c.name }}</a>
                    </li>
                {% endfor %}
                <li class="breadcrumb-item">
                    <a href="{{ product.get_absolute_url }}">{{ product.title }}</a>
                </li>
                <li class="breadcrumb-item">
                    <a href="{% url 'catalogue:reviews-list' product_slug=product.slug product_pk=product.pk %}">{% trans "Reviews" %}</a>
                </li>
            {% endwith %}
            <li class="breadcrumb-item active" aria-current="page">{{ review.title }}</li>
        </ol>
    </nav>
{% endblock %}

{% block headertext %}
    {% trans "Customer reviews" %}
{% endblock  %}

{% block content %}
    <section>
        {% include 'oscar/catalogue/reviews/review_product.html' %}
        {% include 'oscar/catalogue/partials/review.html' %}
    </section>
{% endblock %}
