# Generated by Django 3.2.19 on 2023-07-10 08:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('basket', '0011_json_basket_option'),
    ]

    operations = [
        migrations.AddField(
            model_name='line',
            name='tax_code',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='VAT rate code'),
        ),
    ]
