# -*- coding: utf-8 -*-
# Generated by Django 1.9.11 on 2016-11-20 17:07
from __future__ import unicode_literals

from django.db import migrations
import oscar.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('offer', '0002_auto_20151210_1053'),
    ]

    operations = [
        migrations.AlterField(
            model_name='condition',
            name='proxy_class',
            field=oscar.models.fields.NullCharField(default=None, max_length=255, verbose_name='Custom class'),
        ),
    ]
