# Generated by Django 2.2.9 on 2019-12-20 09:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0017_auto_20190816_0938'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='ancestors_are_public',
            field=models.BooleanField(db_index=True, default=True, help_text='The ancestors of this category are public', verbose_name='Ancestor categories are public'),
        ),
        migrations.AddField(
            model_name='category',
            name='is_public',
            field=models.BooleanField(db_index=True, default=True, help_text='Show this category in search results and catalogue listings.', verbose_name='Is public'),
        ),
        migrations.AlterField(
            model_name='product',
            name='is_public',
            field=models.BooleanField(db_index=True, default=True, help_text='Show this product in search results and catalogue listings.', verbose_name='Is public'),
        ),
    ]
