# Generated by Django 3.1.2 on 2021-02-24 07:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('voucher', '0009_make_voucher_names_unique'),
    ]

    operations = [
        migrations.AlterField(
            model_name='voucher',
            name='name',
            field=models.CharField(help_text='This will be shown in the checkout and basket once the voucher is entered', max_length=128, unique=True, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='voucherset',
            name='name',
            field=models.CharField(max_length=100, unique=True, verbose_name='Name'),
        ),
        migrations.RemoveField(
            model_name='voucherset',
            name='offer',
        ),
    ]
