# Generated by Django 5.1.4 on 2025-02-17 14:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("catalogue", "0030_auto_20250217_1247"),
    ]

    operations = [
        migrations.CreateModel(
            name="ProductCategoryHierarchy",
            fields=[
                (
                    "id",
                    models.CharField(
                        max_length=255,
                        primary_key=True,
                    ),
                ),
                ("product_id", models.IntegerField()),
                ("category_id", models.IntegerField()),
            ],
            options={
                "verbose_name": "Product category hierarchy",
                "verbose_name_plural": "Product categories hierarchy",
                "db_table": "catalogue_product_category_hierarchy",
                "managed": False,
            },
        ),
    ]
