# Generated by Django 2.0.7 on 2018-11-15 19:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('offer', '0007_conditionaloffer_exclusive'),
    ]

    operations = [
        migrations.AlterField(
            model_name='conditionaloffer',
            name='priority',
            field=models.IntegerField(db_index=True, default=0, help_text='The highest priority offers are applied first', verbose_name='Priority'),
        ),
        migrations.AlterField(
            model_name='rangeproductfileupload',
            name='date_uploaded',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='Date Uploaded'),
        ),
    ]
