{% extends 'oscar/dashboard/layout.html' %}
{% load i18n %}

{% block title %}
    {% trans "Shipping address" %} | {% trans "Order" %} {{ order.number }} | {{ block.super }}
{% endblock %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item"><a href="{% url 'dashboard:index' %}">{% trans "Dashboard" %}</a></li>
            <li class="breadcrumb-item"><a href="{% url 'dashboard:order-list' %}">{% trans "Orders" %}</a></li>
            <li class="breadcrumb-item">
                <a href="{% url 'dashboard:order-detail' number=order.number %}">{{ order.number }}</a>
            </li>
            <li class="breadcrumb-item active" aria-current="page">{% trans "Edit shipping address" %}</li>
        </ol>
    </nav>
{% endblock %}

{% block headertext %}
    {% blocktrans %}Edit shipping address for order {{ order.number }}{% endblocktrans %}
{% endblock  %}

{% block content %}

    <form id="order_shipping_address_form" method="post" class="card card-body">
        {% csrf_token %}
        {% include "oscar/dashboard/partials/form_fields.html" with form=form style='horizontal' %}
        <div class="form-actions">
            <button type="submit" class="btn btn-primary" data-loading-text="{% trans 'Updating...' %}">{% trans "Update address" %}</button>
            {% trans "or" %} <a href="{% url 'dashboard:order-detail' number=order.number %}">{% trans "cancel" %}</a>
        </div>
    </form>

{% endblock content %}
