{% extends 'oscar/dashboard/layout.html' %}
{% load i18n %}

{% block body_class %}{{ block.super }} users{% endblock %}

{% block title %}
    {% blocktrans with id=alert.id %}Update alert #{{ id }}{% endblocktrans %} | {{ block.super }}
{% endblock %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item"><a href="{% url 'dashboard:index' %}">{% trans "Dashboard" %}</a></li>
            <li class="breadcrumb-item"><a href="{% url 'dashboard:user-alert-list' %}">{% trans "Product alerts" %}</a></li>
            <li class="breadcrumb-item active" aria-current="page">{% blocktrans with id=alert.id %}Alert #{{ id }}{% endblocktrans %}</li>
        </ol>
    </nav>
{% endblock %}

{% block headertext %}
    {% blocktrans with id=alert.id %}Product alert #{{ id }}{% endblocktrans %}
{% endblock %}

{% block dashboard_content %}
    <div class="table-header">
        <h2 class="float-left">{% blocktrans with id=alert.id %}Product alert #{{ id }}{% endblocktrans %}</h2>
        <div class="float-right">
            <form id="update_alert_form" method="post" class="form-inline">
                {% csrf_token %}
                {% include 'oscar/dashboard/partials/form_fields_inline.html' %}
                <button type='submit' class="btn btn-primary" data-loading-text="{% trans 'Saving...' %}">{% trans "Save" %}</button>
                <a href="{% url 'dashboard:user-alert-list' %}" class="btn btn-secondary">{% trans "cancel" %}</a>
            </form>
        </div>
    </div>
    {% include 'oscar/dashboard/users/alerts/partials/alert.html' %}
{% endblock dashboard_content %}
