{% extends 'oscar/dashboard/layout.html' %}
{% load category_tags %}
{% load i18n %}

{% block body_class %}{{ block.super }} create-page partner{% endblock %}

{% block title %}
    {{ title }} | {% trans "Partners" %} | {{ block.super }}
{% endblock %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item"><a href="{% url 'dashboard:index' %}">{% trans "Dashboard" %}</a></li>
            <li class="breadcrumb-item"><a href="{% url 'dashboard:partner-list' %}">{% trans "Partners" %}</a></li>
            <li class="breadcrumb-item active" aria-current="page">{{ title }}</li>
        </ol>
    </nav>
{% endblock %}

{% block headertext %}{{ title }}{% endblock %}

{% block dashboard_content %}
    <div class="card card-body">
        {% include 'oscar/dashboard/partials/form.html' with form=form %}
    </div>
{% endblock dashboard_content %}
