# Generated by Django 5.2 on 2025-10-01 14:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("order", "0019_order_analytics_tracked"),
    ]

    operations = [
        migrations.AlterField(
            model_name="order",
            name="analytics_tracked",
            field=models.BooleanField(default=False),
        ),
    ]
