{% extends 'oscar/dashboard/reports/index.html' %}
{% load currency_filters %}
{% load i18n %}

{% block report %}
<table class="table table-striped table-bordered table-hover">
    <tr>
        <th>{% trans "Product" %}</th>
        <th>{% trans "Views" %}</th>
        <th>{% trans "Basket additions" %}</th>
        <th>{% trans "Purchases" %}</th>
    </tr>
    {% for product in objects %}
    <tr>
        <td>
        <a href="{% url 'dashboard:catalogue-product' product.product.id %}">{{ product.product }}</a>
        </td>
        <td>{{ product.num_views }}</td>
        <td>{{ product.num_basket_additions }}</td>
        <td>{{ product.num_purchases }}</td>
    </tr>
    {% endfor %}
</table>
{% include "oscar/dashboard/partials/pagination.html" %}
{% endblock %}
