{% extends "oscar/communication/emails/base.html" %}
{% load i18n %}
{% load url_tags %}

{% block tbody %}
<tr>
    <td class="content-block">
         {% blocktrans with email=new_email %}You're receiving this email because your email address has been changed to <strong>{{ email }}</strong>.{% endblocktrans %}
    </td>
</tr>

<tr>
    <td class="content-block">
        {% trans "If it wasn't you who changed it, please reset your password immediately and correct your email address:" %}
    </td>
</tr>

<tr>
    <td class="content-block">
        <a href="{% absolute_url site.domain reset_url %}" class="btn-primary">{% trans "Reset password" %}</a>
    </td>
</tr>

<tr>
    <td class="content-block">
        {% trans "If it was you who changed the email address, you can ignore this email." %}
    </td>
</tr>

{% endblock %}
