{% extends "oscar/layout.html" %}

{% load i18n %}

{% block title %}
    {% trans 'Register' %} | {{ block.super }}
{% endblock %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item">
                <a href="{{ homepage_url }}">{% trans 'Home' %}</a>
            </li>
            <li class="breadcrumb-item active" aria-current="page">{% trans 'Register' %}</li>
        </ol>
    </nav>
{% endblock %}

{% block headertext %}
    {% trans "Register" %}
{% endblock %}

{% block content %}
    <div class="row">
        <div class="col-sm-6 register_form">
            <form id="register_form" method="post">
                {% csrf_token %}
                {% include "oscar/partials/form_fields.html"  %}
                <button name="registration_submit" type="submit" value="Register" class="btn btn-primary btn-lg" data-loading-text="{%trans 'Registering...' %}">{% trans 'Register' %}</button>
                {% if cancel_url %}
                    {% trans "or" %} <a href="{{ cancel_url }}">{% trans "cancel" %}</a>.
                {% endif %}
            </form>
        </div>
    </div>
{% endblock content %}
