{% extends "oscar/base.html" %}

{% block extrahead %}
    {{ block.super }}

    <meta name="robots" content="NONE,NOARCHIVE">
{% endblock %}


{% block tracking %}
    {# By default there is no analytics tracking in the dashboard #}
{% endblock %}
