# Generated by Django 3.0.9 on 2020-08-28 09:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('offer', '0009_auto_20200801_0817'),
    ]

    operations = [
        migrations.AddField(
            model_name='conditionaloffer',
            name='combinations',
            field=models.ManyToManyField(blank=True, help_text='Select other non-exclusive offers that this offer can be combined with on the same items', limit_choices_to={'exclusive': False}, related_name='in_combination', to='offer.ConditionalOffer'),
        ),
    ]
