{% load currency_filters %}
{% load i18n %}
{% load purchase_info_tags %}

{% purchase_info_for_product request product as session %}

{% if session.price.exists %}
    {% if session.price.excl_tax == 0 %}
        <p class="price_color">{% trans "Free" %}</p>
    {% elif session.price.is_tax_known %}
        <p class="price_color">{{ session.price.incl_tax|currency:session.price.currency }}</p>
    {% else %}
        <p class="price_color">{{ session.price.excl_tax|currency:session.price.currency }}</p>
    {% endif %}
{% else %}
    <p class="price_color">&nbsp;</p>
{% endif %}
{% if session.price.exists %}
<p class="{{ session.availability.code }} availability">
    {% if session.availability.is_available_to_buy %}
        <i class="fas fa-check-circle"></i>
    {% else %}
        <i class="fas fa-ban"></i>
    {% endif %}
    {% if verbose %}
        {{ session.availability.message }}
    {% else %}
        {{ session.availability.short_message }}
    {% endif %}
</p>
{% else %}
    <i class="fas fa-ban"></i>
    {% trans 'Unavailable' %}
{% endif %}
