# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-04-18 20:32
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('voucher', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='voucher',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='voucherapplication',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True),
        ),
    ]
