{% extends 'oscar/layout.html' %}
{% load i18n %}

{% block title %}{% trans 'Password reset complete' %} | {{ block.super }}{% endblock %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item">
                <a href="{{ homepage_url }}">{% trans 'Home' %}</a>
            </li>
            <li class="breadcrumb-item">
                <a href="{% url 'password-reset' %}">{% trans 'Password reset' %}</a>
            </li>
            <li class="breadcrumb-item active" aria-current="page">{% trans 'Complete' %}</li>
        </ol>
    </nav>
{% endblock %}

{% block headertext %}{% trans "Password reset complete" %}{% endblock %}

{% block content %}
    <p>{% trans "Your password has been set.  You may go ahead and log in now." %}</p>
    <p><a href="{{ login_url }}" class="btn btn-lg btn-primary">{% trans 'Log in' %}</a></p>
{% endblock %}
