{% extends 'oscar/dashboard/offers/step_form.html' %}
{% load i18n %}

{% block summary %}
    {% include 'oscar/dashboard/offers/summary.html' %}
{% endblock %}

{% block progress %}
    {% include 'oscar/dashboard/offers/progress.html' with step=4 %}
{% endblock %}

{% block form_actions_buttons %}
    <button class="btn btn-primary" type="submit" data-loading-text="{% trans 'Saving...' %}">{% trans "Save this offer" %}</button>
{% endblock %}
