# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-03-27 07:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0015_product_is_public'),
    ]

    operations = [
        migrations.AlterField(
            model_name='productattributevalue',
            name='value_boolean',
            field=models.NullBooleanField(db_index=True, verbose_name='Boolean'),
        ),
        migrations.AlterField(
            model_name='productattributevalue',
            name='value_date',
            field=models.DateField(blank=True, db_index=True, null=True, verbose_name='Date'),
        ),
        migrations.AlterField(
            model_name='productattributevalue',
            name='value_datetime',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='DateTime'),
        ),
        migrations.AlterField(
            model_name='productattributevalue',
            name='value_float',
            field=models.FloatField(blank=True, db_index=True, null=True, verbose_name='Float'),
        ),
        migrations.AlterField(
            model_name='productattributevalue',
            name='value_integer',
            field=models.IntegerField(blank=True, db_index=True, null=True, verbose_name='Integer'),
        ),
    ]
