{% load i18n %}
{% blocktrans with range_name=range.name filename=upload.filepath %}
    <p>File <strong>{{ filename }}</strong> processed.</p>
{% endblocktrans %}

<ul>
    {% if upload.num_new_skus %}
        <li>
            {% blocktrans with action=action count n=upload.num_new_skus %}
                <strong>{{ n }}</strong> product has been {{ action }}
            {% plural %}
                <strong>{{ n }}</strong> products have been {{ action }}
            {% endblocktrans %}
        </li>
    {% endif %}
    {% if upload.num_duplicate_skus %}
        <li>
            {% blocktrans count n=upload.num_duplicate_skus %}
                There was <strong>{{ n }}</strong> duplicate product in the file
            {% plural %}
                There were <strong>{{ n }}</strong> duplicate products in the file
            {% endblocktrans %}
        </li>
    {% endif %}
    {% if upload.num_unknown_skus %}
        <li>
            {% blocktrans count n=upload.num_unknown_skus %}
                There was <strong>{{ n }}</strong> product in the file that couldn't be found in the catalogue
            {% plural %}
                There were <strong>{{ n }}</strong> products in the file that couldn't be found in the catalogue
            {% endblocktrans %}
        </li>
    {% endif %}
</ul>
<p>
    <a href="{% url 'dashboard:range-update' pk=range.pk %}" class="btn btn-success">{% trans "Edit range metadata" %}</a>
    <a href="{% url 'dashboard:range-list' %}" class="btn btn-success">{% trans "Return to ranges homepage" %}</a>
</p>
