# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-03-23 15:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payment', '0002_auto_20141007_2032'),
    ]

    operations = [
        migrations.AlterField(
            model_name='source',
            name='reference',
            field=models.CharField(blank=True, max_length=255, verbose_name='Reference'),
        ),
    ]
