{% load django_tables2 %}
{% load i18n %}
<div class="btn-toolbar">
    <div class="dropdown">
        <button class="btn btn-secondary dropdown-toggle" type="button" id="dropdownMenuButton" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
            {% trans "Actions" %}
        </button>
        <div class="dropdown-menu dropdown-menu-right" aria-labelledby="dropdownMenuButton">
            <a class="dropdown-item" href="{% url 'dashboard:catalogue-option-update' pk=record.id %}{% querystring %}">
                {% trans "Edit Option" %}
            </a>
            <a class="dropdown-item" href="{% url 'dashboard:catalogue-option-delete' pk=record.id %}{% querystring %}">
                {% trans "Delete" %}
            </a>
        </div>
    </div>
</div>
