<footer class="container">
    {% block footer %}
        {% comment %}
            Could be used for displaying links to privacy policy, terms of service, etc.
         {% endcomment %}
    {% endblock %}
</footer>
