{% extends "oscar/customer/baseaccountpage.html" %}
{% load i18n %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item">
                <a href="{{ homepage_url }}">{% trans 'Home' %}</a>
            </li>
            <li class="breadcrumb-item">
                <a href="{% url 'customer:summary' %}">{% trans 'Account' %}</a>
            </li>
            <li class="breadcrumb-item">
                <a href="{% url 'customer:wishlists-list' %}">{% trans 'Wish Lists' %}</a>
            </li>
            <li class="breadcrumb-item">
                <a href="{% url 'customer:wishlists-detail' key=wishlist.key %}">{{ wishlist.name }}</a>
            </li>
            <li class="breadcrumb-item active" aria-current="page">{% trans "Delete?" %}</li>
        </ol>
    </nav>
{% endblock %}

{% block tabcontent %}
    <form method="post">
        {% csrf_token %}
        <p>
            {% blocktrans with name=wishlist.name %}
                Are you sure you want to delete {{ name }}?
            {% endblocktrans %}
        </p>

        <div class="form-group">
            <button type="submit" class="btn btn-danger btn-lg" data-loading-text="{% trans 'Deleting...' %}">{% trans 'Delete' %}</button> {% trans 'or' %} <a href="{% url 'customer:wishlists-list' %}">{% trans 'cancel' %}</a>
        </div>
    </form>
{% endblock tabcontent %}

