{% extends 'oscar/dashboard/layout.html' %}

{% load i18n %}

{% block body_class %}{{ block.super }} create-page users{% endblock %}

{% block title %}
    {% blocktrans with id=alert.id %}Alert #{{ id }}{% endblocktrans %} | {{ block.super }}
{% endblock %}

{% block breadcrumbs %}
    <nav aria-label="breadcrumb">
        <ol class="breadcrumb">
            <li class="breadcrumb-item"><a href="{% url 'dashboard:index' %}">{% trans "Dashboard" %}</a></li>
            <li class="breadcrumb-item"><a href="{% url 'dashboard:user-alert-list' %}">{% trans "Product alerts" %}</a></li>
            <li class="breadcrumb-item active" aria-current="page">{% blocktrans with id=alert.id %}Alert #{{ id }}{% endblocktrans %}</li>
        </ol>
    </nav>
{% endblock %}

{% block headertext %}
    {% blocktrans with id=alert.id %}Delete alert #{{ id }}?{% endblocktrans %}
{% endblock %}

{% block dashboard_content %}
    <div class="table-header">
        <h2>{% blocktrans with id=alert.id %}Product alert #{{ id }}{% endblocktrans %}</h2>
    </div>
    <div class="card card-body bg-light">
        {% include 'oscar/dashboard/users/alerts/partials/alert.html' %}
        <form id="delete_alert_form" method="post">
            {% csrf_token %}
            <div class="form-actions">
                <p>{% trans "Are you sure that you want to delete this alert?" %}</p>
                <button type='submit' class="btn btn-danger" data-loading-text="{% trans 'Deleting...' %}">{% trans "Delete" %}</button> {% trans "or" %}
                <a href="{% url 'dashboard:user-alert-list' %}">{% trans "cancel" %}</a>
            </div>
        </form>
    </div>
{% endblock dashboard_content %}
