{% extends "oscar/checkout/checkout.html" %}
{% load i18n %}

{% block title %}
    {% trans "Edit address" %} | {{ block.super }}
{% endblock title %}

{% block order_contents %}{% endblock %}
{% block shipping_method %}{% endblock %}
{% block payment_method %}{% endblock %}
{% block payment_details %}{% endblock %}

{% block checkout_nav %}
    {% include 'oscar/checkout/nav.html' with step=1 %}
{% endblock %}

{% block checkout_title %}{% trans "Edit address" %}{% endblock %}

{% block shipping_address %}
    <div class="card card-body bg-light">
        <form id="update_user_address" method="post">
            {% csrf_token %}
            {% include "oscar/partials/form_fields.html" with form=form style='horizontal' %}
            <div class="form-group row">
                <div class="offset-sm-4 col-sm-8">
                    <button type="submit" class="btn btn-lg btn-primary" data-loading-text="{% trans 'Saving...' %}">{% trans "Save" %}</button>
                    {% trans "or" %} <a href="{% url 'checkout:shipping-address' %}">{% trans "cancel" %}</a>.
                </div>
            </div>
        </form>
    </div>
{% endblock shipping_address %}
