{% load i18n %}

{% block message_content %}
{% blocktrans with title=line.product.get_title %}
<strong>{{ title }}</strong> has been moved back to your basket.
{% endblocktrans %}
{% endblock message_content %}

{% block analytics_scripts %}{% endblock analytics_scripts %}
