{% extends "oscar/dashboard/base.html" %}
{% load i18n %}
{% load static %}
{% load widget_tweaks %}
{% block styles %}
    <link rel="stylesheet" type="text/css" href="{% static "oscar/css/dashboard.css" %}" />
{% endblock %}

{% block body_class %}dashboard-login{% endblock %}

{% block layout %}
    <div class="card col-sm-4 mx-auto mt-5">
        <div class="card-body">
        <form method="post" novalidate id="dashboard_login_form">
            <h1>{{ shop_name }}</h1>
            <hr>

            {% if form.errors %}
            <div class="alert alert-danger">
            {% trans "Please enter a correct username and password. Note that both fields may be case-sensitive." %}
        </div>
            {% endif %}

            <div class="form-group">
                {{ form.username.label_tag }}
                {% render_field form.username class="form-control" %}
            </div>
            <div class="form-group">
                {{ form.password.label_tag }}
                {% render_field form.password class="form-control" %}
            </div>

            <input type="submit" class="btn btn-primary btn-lg" value="{% trans "Login" %}">
            <input type="hidden" name="next" value="{{ next }}">
            {% csrf_token %}
        </form>
        </div>
    </div>
{% endblock %}
{% block onbodyload %}{% endblock %}
