# Generated by Django 2.2.10 on 2020-07-24 08:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0009_surcharge'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='line',
            name='est_dispatch_date',
        ),
        migrations.RemoveField(
            model_name='line',
            name='unit_cost_price',
        ),
        migrations.RemoveField(
            model_name='line',
            name='unit_retail_price',
        ),
    ]
