{% load image_tags %}

<div class="row">
    <div class="col-sm-2">
        <div class="image_container w-100">
            {% with image=product.primary_image %}
                {% oscar_thumbnail image.original "x155" upscale=False as thumb %}
                <a href="{{ product.get_absolute_url }}">
                    <img src="{{ thumb.url }}" alt="{{ product.get_title }}" class="img-thumbnail w-auto mx-auto my-0">
                </a>
            {% endwith %}
        </div>
    </div>
    <div class="col-sm-10">
        <h2><a href="{{ product.get_absolute_url }}">{{ product.get_title }}</a></h2>
        {% include "oscar/catalogue/partials/stock_record.html" %}
        {% include "oscar/catalogue/reviews/partials/review_stars.html" %}
    </div>
</div>

<hr>
