{% extends "oscar/customer/baseaccountpage.html" %}


{% load i18n %}

{% block extra_breadcrumbs %}
    <li class="breadcrumb-item">
        <a href="{% url 'customer:address-list' %}">{% trans 'Address book' %}</a>
    </li>
{% endblock %}

{% block tabcontent %}
    {% with style='horizontal' %}
        {% include "oscar/partials/form.html" %}
    {% endwith %}
{% endblock tabcontent %}
