# Generated by Django 2.0 on 2017-12-12 04:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('voucher', '0002_auto_20170418_2132'),
    ]

    operations = [
        migrations.AlterField(
            model_name='voucher',
            name='offers',
            field=models.ManyToManyField(limit_choices_to={'offer_type': 'Voucher'}, related_name='vouchers', to='offer.ConditionalOffer', verbose_name='Offers'),
        ),
    ]
