//// TABLE CHANGES
//// -----------

.table-container {
  width: 100%;
}

.table {
  background-color: $white;
}

.table thead tr th.asc span:before {
  font-family: $font-family-icons;
  font-weight: 900;
  content: "\f077";
}

.table thead tr th.desc span:before {
  font-family: $font-family-icons;
  font-weight: 900;
  content: "\f078";
}

.table-header {
  display: flow-root;
}

caption,
.table-header {
  h2, h3 {
    font-weight: bold;
    margin: 0;

    > i {
      padding-right: 10px;
    }
  }

  font-weight: bold;
  border: 1px solid $gray-400;
  border-bottom: none;
  color: $gray-900;
  padding: 6px 10px;
  text-align: left;
  @include border-radius(3px 3px 0 0);
  @include gradient-y($gray-100, $gray-200);

  .btn-toolbar {
    margin-top: 0;
    margin-bottom: 0;
  }

  > i {
    padding-right: 10px;
  }
}

caption {
  caption-side: top;
}
