{% extends "oscar/communication/emails/base.html" %}
{% load i18n %}
{% load url_tags %}

{% block tbody %}
<tr>
    <td class="content-block">
        {% blocktrans with name=site.name %}You're receiving this e-mail because you requested a password reset for your user account at {{ name }}.{% endblocktrans %}
    </td>
</tr>

<tr>
    <td class="content-block">
        {% trans "Please go to the following page and choose a new password:" %}
    </td>
</tr>

<tr>
    <td class="content-block">
        <a href="{% absolute_url site.domain reset_url %}" class="btn-primary">{% trans "Reset password" %}</a>
    </td>
</tr>

{% endblock %}
