# Generated by Django 3.0.8 on 2020-08-01 07:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0010_auto_20200724_0909'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='orderdiscount',
            options={'ordering': ['pk'], 'verbose_name': 'Order Discount', 'verbose_name_plural': 'Order Discounts'},
        ),
        migrations.AlterModelOptions(
            name='ordernote',
            options={'ordering': ['-date_updated'], 'verbose_name': 'Order Note', 'verbose_name_plural': 'Order Notes'},
        ),
    ]
